==> thx_contrib-0.0.1-py3-none-any.whl: thx/contrib/pdf/utils.py <==
# -*- coding: utf-8 -*-

import io
from pathlib import Path
from typing import Any, Dict, List, Optional

import httpx
from django.core.files.base import ContentFile
from django.template import Context, Template
from django.template.loader import render_to_string
from pikepdf import AttachedFileSpec, Pdf
from pyhanko.pdf_utils.reader import PdfFileReader
from PyPDF2 import PdfReader, PdfReader as PyPdfFileReader, PdfWriter, Transformation
from weasyprint import CSS, HTML
from weasyprint.fonts import FontConfiguration


def _render_to_string(template_str, context: dict):
    return Template(template_str).render(Context(context))


def render_to_pdf(
    html_template: Optional[str],
    html_template_filename: Optional[str] = None,
    css_template_filename: Optional[str] = None,
    html_context: Optional[Dict] = None,
    target: Optional[Any] = None,
) -> HTML:
    """Render html template to PDF"""
    if html_template:
        if html_template_filename:
            raise ValueError("Use html_template or html_template_filename.")

    font_config = FontConfiguration()
    stylesheets = []

    if html_template:
        html_template = _render_to_string(html_template, html_context)
    else:
        html_template = render_to_string(html_template_filename, html_context)

    if css_template_filename:
        css = CSS(string=render_to_string(css_template_filename))
        stylesheets.append(css)

    html = HTML(string=html_template)
    if target:
        html.write_pdf(target, stylesheets=stylesheets, font_config=font_config, presentational_hints=True)
    return html


def merge_binary_pdf(target, file_list: List[io.BytesIO]):
    merger = Pdf.new()
    for file in file_list:
        file = ContentFile(file.read())
        src = Pdf.open(file)
        merger.pages.extend(src.pages)
    merger.save(target)
    target.seek(0)


def check_signed_document(document_path: str) -> bool:
    with open(document_path, 'rb') as doc:
        try:
            r = PdfFileReader(doc)
        except:
            r = PdfFileReader(doc, strict=False)
        if r.embedded_signatures:
            return True
    return False


def add_watermark_to_pdf_pages(
    file_path: str,
    watermark_file_path: str,
    page_start: Optional[int] = 0,
    page_end: Optional[int] = None,
    page_step: Optional[int] = 1
):
    with open(file_path, "rb") as input_file, open(watermark_file_path, "rb") as watermark_file:
        input_pdf = PyPdfFileReader(input_file)
        watermark_pdf = PyPdfFileReader(watermark_file)
        watermark_page = watermark_pdf.pages[0]
        pdf_output = PdfWriter()
        if not page_end:
            page_end = len(input_pdf.pages)
        for i in range(page_start, page_end, page_step):
            pdf_page = input_pdf.pages[i]
            mediabox = pdf_page.mediabox
            pdf_page.merge_page(watermark_page)
            pdf_page.mediabox = mediabox
            pdf_output.add_page(pdf_page)
        with open(file_path, "wb") as out_file:
            pdf_output.write(out_file)


def add_attachments_to_pdf(file_path: str, file_path_list: List[str]):
    pdf = Pdf.open(file_path, allow_overwriting_input=True)
    for index, attachment_path in enumerate(file_path_list):
        filespec = AttachedFileSpec.from_filepath(pdf, Path(attachment_path))
        pdf.attachments[f'attachment{index}.pdf'] = filespec
        pdf.save()


def convert_to_pdf(
    files: List[str],
    pod_converter_base_url: str,
    pod_merge_endpoint: str,
    pod_convert_endpoint: str,
    pod_timeout:  Optional[int] = 10,
    output_filename: Optional[str] = None,
    output_format_orig: Optional[str] = 'pdf',
    merge: Optional[bool] = False
):
    """Convert input to pdf. If many files are handled one single merged file is returned

    :arg files: filename or open file handler (can be a list/tuple)
    :arg pod_converter_base_url: base url for POD converter
    :arg pod_merge_endpoint: POD API endpoint for PDF merge
    :arg pod_convert_endpoint: POD API endpoint for PDF conversion
    :arg pod_timeout: timeout for the connection to the server (default 10)
    :arg output_filename: name of the output file (optional)
    :arg output_format_orig: format for output file (optional)
    :arg merge: (boolean). Suggest that we need to generate an output that is the merge
       of input files (that need to be ``.pdf``)
    """
    if not isinstance(files, (list, tuple)):
        files = [files]

    files_dict = {}
    for j, file in enumerate(files):
        if isinstance(file, str):
            files_dict[f'file{j:02}'] = open(file, 'rb')
        else:
            files_dict[f'file{j:02}'] = file

    if merge:
        POD_CONVERTER_URL = pod_converter_base_url + pod_merge_endpoint
    else:
        POD_CONVERTER_URL = f'{pod_converter_base_url}{pod_convert_endpoint}?output_format={output_format_orig}'

    rh = httpx.post(POD_CONVERTER_URL, files=files_dict, timeout=pod_timeout, data={"format": output_format_orig})

    if output_filename:
        with open(output_filename, 'wb') as f:
            f.write(rh.content)
    return rh


def add_y_margin_to_pdf(
    file_path: str,
    margin: Optional[int] = 40,
    scale: Optional[float] = 0.95,
    page_start: Optional[int] = 0,
    page_end: Optional[int] = None,
    page_step: Optional[int] = 1
):
    writer = PdfWriter()
    with open(file_path, 'rb') as f:
        p = PdfReader(f)
        if not page_end:
            page_end = len(p.pages)
        for i in range(page_start, page_end, page_step):
            page = p.pages[i]
            page.add_transformation(Transformation().scale(scale).translate(0, margin))
            writer.add_page(page)

    with open(file_path, 'wb') as out_file:
        writer.write(out_file)

==> thx_contrib-0.0.4-py3-none-any.whl: thx/contrib/pdf/utils.py <==
# -*- coding: utf-8 -*-

import io
from pathlib import Path
from typing import Any, Dict, List, Optional

import httpx
from django.core.files.base import ContentFile
from django.template import Context, Template
from django.template.loader import render_to_string
from pikepdf import AttachedFileSpec, Pdf
from pyhanko.pdf_utils.reader import PdfFileReader
from PyPDF2 import PdfReader, PdfReader as PyPdfFileReader, PdfWriter, Transformation
from weasyprint import CSS, HTML
from weasyprint.fonts import FontConfiguration


def _render_to_string(template_str, context: dict):
    return Template(template_str).render(Context(context))


def render_to_pdf(
    html_template: Optional[str],
    html_template_filename: Optional[str] = None,
    css_template_filename: Optional[str] = None,
    html_context: Optional[Dict] = None,
    target: Optional[Any] = None,
) -> HTML:
    """Render html template to PDF"""
    if html_template:
        if html_template_filename:
            raise ValueError("Use html_template or html_template_filename.")

    font_config = FontConfiguration()
    stylesheets = []

    if html_template:
        html_template = _render_to_string(html_template, html_context)
    else:
        html_template = render_to_string(html_template_filename, html_context)

    if css_template_filename:
        css = CSS(string=render_to_string(css_template_filename))
        stylesheets.append(css)

    html = HTML(string=html_template)
    if target:
        html.write_pdf(target, stylesheets=stylesheets, font_config=font_config, presentational_hints=True)
    return html


def merge_binary_pdf(target, file_list: List[io.BytesIO]):
    merger = Pdf.new()
    for file in file_list:
        file = ContentFile(file.read())
        src = Pdf.open(file)
        merger.pages.extend(src.pages)
    merger.save(target)
    target.seek(0)


def check_signed_document(document_path: str) -> bool:
    with open(document_path, 'rb') as doc:
        try:
            r = PdfFileReader(doc)
        except:
            r = PdfFileReader(doc, strict=False)
        if r.embedded_signatures:
            return True
    return False


# TODO: check for a better solution
def add_watermark_to_pdf_pages(
    file_path: str,
    watermark_file_path: str = None,
    landscape_watermark_file_path: str = None,
    page_start: Optional[int] = 0,
    page_end: Optional[int] = None,
    page_step: Optional[int] = 1
):
    if not watermark_file_path and not landscape_watermark_file_path:
        raise ValueError("watermark_file_path or landscape_watermark_file_path is required")

    if not watermark_file_path:
        watermark_file_path = landscape_watermark_file_path
    if not landscape_watermark_file_path:
        landscape_watermark_file_path = watermark_file_path

    with open(file_path, "rb") as input_file, \
         open(watermark_file_path, "rb") as watermark_file, \
         open(landscape_watermark_file_path, "rb") as landscape_watermark_file:

        watermark_pdf = PyPdfFileReader(watermark_file)
        watermark_page = watermark_pdf.pages[0]
        landscape_watermark_pdf = PyPdfFileReader(landscape_watermark_file)
        landscape_watermark_page = landscape_watermark_pdf.pages[0]
        input_pdf = PyPdfFileReader(input_file)
        pdf_output = PdfWriter()
        if not page_end:
            page_end = len(input_pdf.pages)
        for i in range(page_start, page_end, page_step):
            pdf_page = input_pdf.pages[i]
            mediabox = pdf_page.mediabox
            if mediabox.width > mediabox.height:
                pdf_page.merge_page(landscape_watermark_page)
            else:
                pdf_page.merge_page(watermark_page)
            pdf_page.mediabox = mediabox
            pdf_output.add_page(pdf_page)
        with open(file_path, "wb") as out_file:
            pdf_output.write(out_file)


def add_attachments_to_pdf(file_path: str, file_path_list: List[str]):
    pdf = Pdf.open(file_path, allow_overwriting_input=True)
    for index, attachment_path in enumerate(file_path_list):
        filespec = AttachedFileSpec.from_filepath(pdf, Path(attachment_path))
        pdf.attachments[f'attachment{index}.pdf'] = filespec
        pdf.save()


def convert_to_pdf(
    files: List[str],
    pod_converter_base_url: str,
    pod_merge_endpoint: str,
    pod_convert_endpoint: str,
    pod_timeout:  Optional[int] = 10,
    output_filename: Optional[str] = None,
    output_format_orig: Optional[str] = 'pdf',
    merge: Optional[bool] = False
):
    """Convert input to pdf. If many files are handled one single merged file is returned

    :arg files: filename or open file handler (can be a list/tuple)
    :arg pod_converter_base_url: base url for POD converter
    :arg pod_merge_endpoint: POD API endpoint for PDF merge
    :arg pod_convert_endpoint: POD API endpoint for PDF conversion
    :arg pod_timeout: timeout for the connection to the server (default 10)
    :arg output_filename: name of the output file (optional)
    :arg output_format_orig: format for output file (optional)
    :arg merge: (boolean). Suggest that we need to generate an output that is the merge
       of input files (that need to be ``.pdf``)
    """
    if not isinstance(files, (list, tuple)):
        files = [files]

    files_dict = {}
    for j, file in enumerate(files):
        if isinstance(file, str):
            files_dict[f'file{j:02}'] = open(file, 'rb')
        else:
            files_dict[f'file{j:02}'] = file

    if merge:
        pod_converter_url = pod_converter_base_url + pod_merge_endpoint
    else:
        pod_converter_url = f'{pod_converter_base_url}{pod_convert_endpoint}?output_format={output_format_orig}'

    rh = httpx.post(pod_converter_url, files=files_dict, timeout=pod_timeout, data={"format": output_format_orig})

    if output_filename:
        with open(output_filename, 'wb') as f:
            f.write(rh.content)
    return rh


def add_y_margin_to_pdf(
    file_path: str,
    margin: Optional[int] = 40,
    scale: Optional[float] = 0.95,
    page_start: Optional[int] = 0,
    page_end: Optional[int] = None,
    page_step: Optional[int] = 1
):
    writer = PdfWriter()
    with open(file_path, 'rb') as f:
        p = PdfReader(f)
        if not page_end:
            page_end = len(p.pages)
        for i in range(page_start, page_end, page_step):
            page = p.pages[i]
            page.add_transformation(Transformation().scale(scale).translate(0, margin))
            writer.add_page(page)

    with open(file_path, 'wb') as out_file:
        writer.write(out_file)
